@@ -18,16 +18,6 @@ from pagetypes import PAGE_TYPE_UPGRADES
 from Defaults import PAGE_METADATA, \
      TEXTINDEXES, FIELDINDEXES, KEYWORDINDEXES, DATEINDEXES, PATHINDEXES
 
-# copied from Wikis.py
-def _addDTMLMethod(self, id, title='', file=''):
-    id=str(id)
-    title=str(title)
-    ob = DTMLMethod(source_string=file, __name__=id)
-    ob.title = title
-    username = getSecurityManager().getUser().getUserName()
-    ob.manage_addLocalRoles(username, ['Owner'])
-    #ob.setSubOwner('both') #?
-    self._setObject(id, ob)
 
 class PageAdminSupport:
     """
@@ -41,20 +31,26 @@ class PageAdminSupport:
         """
         Update, upgrade, pre-render and re-index all pages and data structures.
 
-        Requires 'Manage properties' permission on the folder.
-        Normally pages are upgraded/pre-rendered as needed when viewed.
+        Normally individual pages are upgraded and pre-rendered on
+        demand, when viewed. An administrator may want to do this for
+        all pages ahead of time, particularly after a zwiki upgrade,
+        to ensure all pages have the latest properties and have been
+        rendered by the latest code, minimizing delay and possible
+        problems later on.
 
-        An administrator may want to call this ahead of time, particularly
-        after a zwiki upgrade, to ensure all pages have the latest
-        properties and have been rendered by the latest code, minimizing
-        delay and possible problems later on. If you don't need to
-        re-render every page, set render=0 to complete much faster on
-        large wikis. Also validates parents, re-catalogs each page, and
-        rebuilds the cached wiki outline.
+        Also installs a wiki catalog if not present, re-indexes each
+        page, validates page parents, and rebuilds the wiki outline
+        cache. Also installs the index_html and standard_error_message
+        dtml methods. XXX split ? 
+
+        You can set render=0 to skip the page pre-rendering part,
+        completing much faster on large wikis.
 
         The optional batch argument forces a commit every N pages.
         This may be useful to get a complete run in large/busy wikis,
-        which can be difficult due to conflict errors, memory usage etc.
+        which can be difficult due to conflict errors or memory usage.
+
+        Requires 'Manage properties' permission on the folder.
         """
         if not self.checkPermission(Permissions.manage_properties,
                                      self.folder()):
@@ -62,15 +58,18 @@ class PageAdminSupport:
              _('You are not authorized to upgrade all pages.') + \
              _('(folder -> Manage properties)'))
         
-        if render: BLATHER('upgrading and pre-rendering all pages:')
-        else: BLATHER('upgrading all pages:')
+        batch = int(batch)
+        if render: BLATHER('upgrading/reindexing/pre-rendering all pages:')
+        else: BLATHER('upgrading/reindexing all pages:')
         starttime = clock()
         n, total = 0, self.pageCount()
+        self.setupCatalog(reindex=0)
         for p in self.pageObjects(): # poor caching (not a problem here)
             n += 1
             try:
                 p.upgrade(REQUEST)
                 p.upgradeId(REQUEST)
+                p.fixPageEncoding()
                 if render:
                     p.preRender(clear_cache=1)
                     msg = 'upgraded and pre-rendered page'
@@ -87,6 +86,7 @@ class PageAdminSupport:
                 get_transaction().commit()
 
         self.updateWikiOutline()
+        self.setupDtmlMethods()
         endtime = clock()
         BLATHER('upgrade complete, %d pages processed in %fs, %f pages/s' \
                 %(n, endtime-starttime, n/(endtime-starttime)))
@@ -126,6 +126,12 @@ class PageAdminSupport:
                 'upgradeId for "%s" (%s) failed - does %s already exist ?' \
                 % (self.pageName(),self.getId(),self.canonicalIdFrom(name)))
 
+    def newPageTypeIdFor(self,oldpagetypeid):
+        """
+        Find the best modern equivalent for some arbitrary old page type.
+        """
+        return PAGE_TYPE_UPGRADES.get(oldpagetypeid, self.defaultPageType())
+
     # allow extra actions to be added to this method
     # upgrade hooks return non-null if the page object was changed
     # they should be fast as upgrade is called on each page view
@@ -175,8 +181,9 @@ class PageAdminSupport:
             changed = 1
 
         # upgrade old page types
-        if self.pageTypeId() in PAGE_TYPE_UPGRADES.keys():
-            self.setPageType(PAGE_TYPE_UPGRADES[self.pageTypeId()])
+        pagetype = self.pageTypeId()
+        if pagetype in PAGE_TYPE_UPGRADES.keys():
+            self.setPageType(self.newPageTypeIdFor(pagetype))
             # clear render cache; don't bother prerendering just now
             self.clearCache()
             changed = 1
@@ -205,11 +212,11 @@ class PageAdminSupport:
         if not self.creation_time:
             pass
         elif type(self.creation_time) is not StringType:
-            self.creation_time = self.creation_time.ISO()
+            self.creation_time = self.creation_time.toZone('UTC').ISO()
             changed = 1
         elif len(self.creation_time) != 19:
             try: 
-                self.creation_time = DateTime(self.creation_time).ISO()
+                self.creation_time = DateTime(self.creation_time).toZone('UTC').ISO()
                 changed = 1
             except DateTimeSyntaxError:
                 # can't convert to ISO, just leave it be
@@ -286,7 +293,7 @@ class PageAdminSupport:
         """
         Install some default wiki pages to help get a wiki started.
         """
-        # copied from Wikis.py
+        # copied from ...
         dir = package_home(globals()) + os.sep + 'wikis' + os.sep + 'basic'
         filenames = os.listdir(dir)
         for filename in filenames:
@@ -304,7 +311,7 @@ class PageAdminSupport:
                         text = m.group(3)
                         self.create(name,text=text)
         if REQUEST:
-            REQUEST.RESPONSE.redirect(self.page_url())
+            REQUEST.RESPONSE.redirect(self.pageUrl())
 
     #XXX does this not attempt to create everything ?
     security.declareProtected('Manage properties', 'setupPages')
@@ -316,7 +323,7 @@ class PageAdminSupport:
         It doesn't check if dtml is enabled in the wiki, just
         creates the pages with the default page type.
         """
-        dir = os.path.join(package_home(globals()),'skins','zwiki_standard')
+        dir = os.path.join(package_home(globals()),'skins','zwiki')
         filenames = os.listdir(dir)
         for filename in filenames:
             m = re.search(r'(.+)\.(.+)',filename)
@@ -331,7 +338,7 @@ class PageAdminSupport:
                     text = m.group(3)
                     self.create(name,text=text)
         if REQUEST:
-            REQUEST.RESPONSE.redirect(self.page_url())
+            REQUEST.RESPONSE.redirect(self.pageUrl())
 
     security.declareProtected('Manage properties', 'setupDtmlMethods')
     def setupDtmlMethods(self,REQUEST=None):
@@ -350,13 +357,13 @@ class PageAdminSupport:
         for m in dtmlmethods:
             # avoid acquisition.. self.folder().aq_base won't work
             if m not in ids:
-                _addDTMLMethod(
+                addDTMLMethod(
                     self.folder(),
                     m,
                     title='',
                     file=open(os.path.join(d,m+'.dtml'),'r').read())
         if REQUEST:
-            REQUEST.RESPONSE.redirect(self.page_url())
+            REQUEST.RESPONSE.redirect(self.pageUrl())
 
     security.declareProtected('Manage properties', 'setupProperties')
     def setupProperties(self,REQUEST=None):
@@ -373,7 +380,7 @@ class PageAdminSupport:
         """
         pass
         if REQUEST:
-            REQUEST.RESPONSE.redirect(self.page_url())
+            REQUEST.RESPONSE.redirect(self.pageUrl())
 
     security.declareProtected('Manage properties', 'setupCatalog')
     def setupCatalog(self,REQUEST=None,reindex=1):
@@ -413,14 +420,56 @@ class PageAdminSupport:
             cid = self.catalog().getId()
             for p in self.pageObjects():
                 n = n + 1
-                try:
-                    BLATHER('indexing page #%d %s in %s'%(n,p.id(),cid))
-                    p.index_object(log=0)
-                except:
-                    BLATHER('failed to index page #%d %s: %s' \
-                            % (n,p.id(),formattedTraceback()))
+                BLATHER('indexing page %d %s in %s'%(n,p.id(),cid))
+                p.index_object(log=0)
             BLATHER('indexing complete, %d pages processed' % n)
         if REQUEST:
-            REQUEST.RESPONSE.redirect(self.page_url())
+            REQUEST.RESPONSE.redirect(self.pageUrl())
             
+    def fixPageEncoding(self, FROM='iso8859-1', TO='utf-8'):
+        """
+        Try to fix any character encoding problems in this page's name or text.
+
+        FROM is the encoding that we will try to convert from if there is trouble
+        TO is the encoding we prefer to use in this wiki
+        """
+        try: 
+            # are there characters in the name or text not using our preferred encoding ?
+            t = unicode(self.pageName(),TO)
+            t = unicode(self.text(),TO)
+            return False
+        except UnicodeDecodeError:
+            # yes - convert 
+            log = "converting %s from %s to %s encoding" % (self.pageName(), FROM, TO)
+            BLATHER(log)
+            # this will send mail and set last editor
+            #self.edit(text=unicode(self.text(), FROM).encode(TO),
+            #       title=unicode(self.pageName(), FROM).encode(TO),
+            #       log=log)
+            # this won't
+            self.setText(unicode(self.text(), FROM).encode(TO), REQUEST=REQUEST)
+            # this will (if the name really needs fixing)
+            self.rename(unicode(self.pageName(), FROM).encode(TO), sendmail=0, REQUEST=REQUEST)
+            return True
+
+    def fixAllPagesEncoding(self):
+        for p in self.pageObjects():
+            fixPageEncoding(p)
+        self.REQUEST.RESPONSE.redirect(self.REQUEST['URL1'])
+
 InitializeClass(PageAdminSupport)
+
+
+# misc admin functions.. see also __init__.py
+# things are here and there to avoid circular imports
+
+def addDTMLMethod(self, id, title='', file=''):
+    id=str(id)
+    title=str(title)
+    ob = DTMLMethod(source_string=file, __name__=id)
+    ob.title = title
+    username = getSecurityManager().getUser().getUserName()
+    ob.manage_addLocalRoles(username, ['Owner'])
+    #ob.setSubOwner('both') #?
+    self._setObject(id, ob)
+
