@@ -28,7 +28,6 @@ from Defaults import DISABLE_JAVASCRIPT, LARGE_FILE_SIZE, LEAVE_PLACEHOLDER
 import Permissions
 from Regexps import javascriptexpr, htmlheaderexpr, htmlfooterexpr
 from Utils import get_transaction, BLATHER, parseHeadersBody
-from Views import onlyBodyFrom
 from I18n import _
 
 
@@ -60,13 +59,17 @@ class PageEditingSupport:
         - can handle a rename during page creation. This helps CMF/Plone
         and is occasionally useful.
         
-        - redirects to the new page if appropriate
+        - checks the edit_needs_username property as well as permissions.
+
+        - redirects to the new page or to the denied view if appropriate
+
+        - returns the new page's name or None
 
         """
         if not self.checkSufficientId(REQUEST):
-            return self.denied(
-                _("Sorry, this wiki doesn't allow anonymous edits. Please configure a username in options first."))
-
+            if REQUEST: REQUEST.RESPONSE.redirect(self.pageUrl()+'/denied')
+            return None
+        
         name = unquote(page or pagename)
         id = self.canonicalIdFrom(name)
 
@@ -74,7 +77,7 @@ class PageEditingSupport:
 
         # make a new page object and situate it in the wiki
         # get a hold of it's acquisition wrapper
-        p = self.__class__(source_string='', __name__=id)
+        p = self.__class__(__name__=id)
         # set title now since manage_afterAdd will use it for wiki outline
         p.title = name
         # newid should be the same as id, but don't assume
@@ -120,12 +123,13 @@ class PageEditingSupport:
                 message_id=self.messageIdFromTime(p.creationTime())
                 )
         # and move on
-        if REQUEST is not None:
+        if REQUEST:
             try:
                 u = (REQUEST.get('redirectURL',None) or
                      REQUEST['URL2']+'/'+ quote(p.id()))
                 REQUEST.RESPONSE.redirect(u)
             except KeyError: pass
+        return name
 
     security.declarePublic('isDavLocked')
     def isDavLocked(self):
@@ -157,7 +161,7 @@ class PageEditingSupport:
         oldtext = self.read()
         if not username:
             username = self.usernameFrom(REQUEST)
-            if re.match(r'^[0-9\.\s]*$',username): username = ''
+            if re.match(r'^(?:\d{1,3}\.){3}\d{1,3}$',username): username = ''
         subject_heading = self.cleanupText(subject_heading)
         text = self.cleanupText(text)
         # some subtleties here: we ensure the page comment and mail-out
@@ -373,7 +377,7 @@ class PageEditingSupport:
             # do we have permission ?
             if (not
                 (self.checkPermission(Permissions.Edit, self) or
-                 (self.checkPermission(Permissions.Append, self)
+                 (self.checkPermission(Permissions.Comment, self)
                   and find(new,old) == 0))):
                 raise 'Unauthorized', (
                     _('You are not authorized to edit this ZWiki Page.'))
@@ -400,7 +404,7 @@ class PageEditingSupport:
             return 0
         if (not
             (self.checkPermission(Permissions.Edit, self) or
-             (self.checkPermission(Permissions.Append, self)
+             (self.checkPermission(Permissions.Comment, self)
               and find(self.cleanupText(text),self.read()) == 0))):
             raise 'Unauthorized', (
                 _('You are not authorized to edit this ZWiki Page.'))
@@ -573,7 +577,7 @@ class PageEditingSupport:
                 subject='(renamed)')
 
         BLATHER('rename complete')
-        if REQUEST: REQUEST.RESPONSE.redirect(self.page_url())
+        if REQUEST: REQUEST.RESPONSE.redirect(self.pageUrl())
 
     def changeIdPreservingCreator(self,newid):
         creation_time, creator, creator_ip = \
@@ -654,7 +658,7 @@ class PageEditingSupport:
                 raise 'Unauthorized', (
                     _('You are not authorized to upload files here.'))
             if not (self.checkPermission(Permissions.Edit, self) or
-                    self.checkPermission(Permissions.Append, self)):
+                    self.checkPermission(Permissions.Comment, self)):
                 raise 'Unauthorized', (
                     _('You are not authorized to add a link on this ZWiki Page.'))
             # can we check file's size ?
@@ -790,7 +794,6 @@ class PageEditingSupport:
         Change this page's text.
 
         Does cleanups and triggers pre-rendering and DTML re-parsing.
-        Also inserts purple number NIDs if appropriate.
         """
         self.raw = self.cleanupText(text)
         self.preRender(clear_cache=1)
@@ -816,7 +819,7 @@ class PageEditingSupport:
         def raiseSpamError(reason, verbose_reason):
             BLATHER(('blocked edit from %s (%s) on %s (%s)\n%s\n') % \
                     (ip, username, page, reason, t))
-            raise _("There was a problem: %s. Please contact the site administrator for help." % \
+            raise _("There was a problem: %s" % \
                     (verbose_reason))
             
         # banned link pattern ?
@@ -824,7 +827,7 @@ class PageEditingSupport:
             pat = strip(pat)
             if pat and re.search(pat,t):
                 raiseSpamError(_("banned_links match"),
-                               _("your edit contained a banned link pattern"))
+                               _("your edit contained a banned link pattern. Please contact the site administrator for help."))
 
         # anonymous edit with too many urls ?
         prop = 'max_anonymous_links'
@@ -836,7 +839,19 @@ class PageEditingSupport:
             if max is not None:
                 if len(re.findall(r'https?://',t)) > max:
                     raiseSpamError(_("exceeded max_anonymous_links"),
-                                   _("adding of external links by anonymous authors is restricted"))
+                                   _("adding of external links by unidentified users is restricted. Please back up and remove some of the http urls you added, or contact the site administrator for help."))
+
+        # and a similar check for identified users
+        # XXX simplify ? one property for both ?
+        prop = 'max_identified_links'
+        # we'll handle either an int or string property
+        if (hasattr(self.folder(), prop)):
+            try: max = int(getattr(self.folder(), prop))
+            except ValueError: max = None
+            if max is not None:
+                if len(re.findall(r'https?://',t)) > max:
+                    raiseSpamError(_("exceeded max_identified_links"),
+                                   _("adding of external links is restricted, even for identified users. Please back up and remove some of the http urls you added, or contact the site administrator for help."))
 
     def cleanupText(self, t):
         """
@@ -870,13 +885,22 @@ class PageEditingSupport:
             t = re.sub(javascriptexpr,r'&lt;disabled \1&gt;',t)
 
         # strip out HTML header tags if present
+        def onlyBodyFrom(t):
+            # XXX these can be expensive, for now just skip if there's a problem
+            try:
+                t = re.sub(htmlheaderexpr,'',t)
+                t = re.sub(htmlfooterexpr,'',t)
+            except RuntimeError: pass
+            return t
+            # maybe better, but more inclined to mess with valid text ?
+            #return re.sub(htmlbodyexpr, r'\1', t)
         t = onlyBodyFrom(t)
 
         return t
 
     def setLastEditor(self, REQUEST=None):
         """
-        record my last_editor & last_editor_ip
+        Record last editor info based on the current REQUEST and time.
         """
         if REQUEST:
             self.last_editor_ip = REQUEST.REMOTE_ADDR
@@ -886,7 +910,7 @@ class PageEditingSupport:
             # if we have no REQUEST, at least update last editor
             self.last_editor_ip = ''
             self.last_editor = ''
-        self.last_edit_time = DateTime(time.time()).ISO()
+        self.last_edit_time = DateTime(time.time()).toZone('UTC').ISO()
 
     def hasCreatorInfo(self):
         """
@@ -901,7 +925,7 @@ class PageEditingSupport:
         """
         record my creator, creator_ip & creation_time
         """
-        self.creation_time = DateTime(time.time()).ISO()
+        self.creation_time = DateTime(time.time()).toZone('UTC').ISO()
         if REQUEST:
             self.creator_ip = REQUEST.REMOTE_ADDR
             self.creator = self.usernameFrom(REQUEST)
@@ -1036,6 +1060,29 @@ class PageEditingSupport:
     def defaultPageType(self):
         """This wiki's default page type."""
         return self.allowedPageTypes()[0]
+
+    security.declareProtected(Permissions.Add, 'split')
+    def split(self):
+        """
+        Move this page's major sections to sub-pages, if supported.
+
+        Delegates to the page type; at present only the restructured
+        text page type does this.
+
+        Watch out for confusion with string.split which might arise
+        here and there.
+        """
+        return self.pageType().split(self)
+    
+    security.declareProtected(Permissions.Delete, 'merge')
+    def merge(self):
+        """
+        Merge sub-pages as sections of this page, if supported.
+
+        Delegates to the page type; at present only the restructured
+        text page type does this.
+        """
+        return self.pageType().merge(self)
     
 InitializeClass(PageEditingSupport)
 
