@@ -216,23 +216,25 @@ class ShowSubtopicsProperty:
                 return getattr(self.REQUEST,prop) and 1
             elif hasattr(self.aq_base,prop):
                 return getattr(self,prop) and 1
-            elif self.primaryParent():
-                # poor caching
-                try: return self.primaryParent().subtopicsEnabled() 
-                except:
-                    # experimental: support all-brains
-                    try: return self.primaryParent().getObject().subtopicsEnabled() 
-                    except: # XXX still run into errors here, investigate
-                        BLATHER('DEBUG: error in subtopicsEnabled for %s, primaryParent is: %s'\
-                             % (self.id(),`self.primaryParent()`))
-                        return not (getattr(getattr(self,'REQUEST',None),
-                                            'zwiki_displaymode',
-                                            None) == 'minimal')
             else:
-                #return not (getattr(getattr(self,'REQUEST',None),
-                #                    'zwiki_displaymode',
-                #                    None) == 'minimal')
-                return 1
+                primaryParent = self.primaryParent() # call only once
+                if primaryParent:
+                    try: return primaryParent.subtopicsEnabled()
+                    except AttributeError:
+                        # experimental: support all-brains
+                        try: return primaryParent.getObject().subtopicsEnabled()
+                        except AttributeError: # XXX still run into errors here, investigate
+                            BLATHER('DEBUG: error in subtopicsEnabled for %s, primaryParent is: %s'\
+                                 % (self.id(),`primaryParent`))
+                            return not (getattr(getattr(self,'REQUEST',None),
+                                                'zwiki_displaymode',
+                                                None) == 'minimal')
+                else:
+                    #return not (getattr(getattr(self,'REQUEST',None),
+                    #                    'zwiki_displaymode',
+                    #                    None) == 'minimal')
+                    return 1
+
         else:
             return 0
 
@@ -371,6 +373,10 @@ class OutlineManager:
         Page names may be ids, or fuzzy, even partial. Any which do not
         resolve to an existing page or are duplicates will be ignored.
         """
+        if not self.checkSufficientId(REQUEST):
+            return self.denied(
+                _("Sorry, this wiki doesn't allow anonymous reparenting. Please configure a username in options first."))
+
         oldparents = self.getParents()
         # clean the arguments carefully to avoid parenting anomalies
         # page mgmt form must use pagename field:
@@ -528,13 +534,16 @@ class OutlineManager:
         except: return [] # XXX temp, get rid of
 
     security.declareProtected(Permissions.View, 'siblingsAsList')
-    def siblingsAsList(self):
+    def siblingsAsList(self,include_me=False,sort_alpha=True):
         """
         Return the names of other pages sharing my first parent.
 
         Siblings by my other parents are ignored.
+        Optionally include the current page in the list.
+        Optionally suppress alphabetical sorting of result.
         """
-        return self.wikiOutline().siblings(self.pageName())
+        return self.wikiOutline().siblings(self.pageName(), \
+                include_me=include_me,sort_alpha=sort_alpha)
 
     security.declareProtected(Permissions.View, 'childrenAsList')
     def childrenAsList(self):
@@ -593,7 +602,7 @@ class OutlineManager:
 
     def childrenNesting(self):
         """
-        Return a nesting representing this page's ancestors and children.
+        Return a nesting representing this page's children.
         """
         return self.wikiOutline().children(self.pageName())
 
@@ -642,12 +651,37 @@ class OutlineRendering:
                        self.formatWikiname(i)))
             else:
                 combos.append(i)
-        here = (here and unquote(here)) or self.pageName()
+        # this view is often invoked via a default page, not the current one,
+        # to reduce bot traffic (see contentsUrl). Try to figure out the
+        # current page for "you are here".
+        here = unquote(here or self.referringPageName() or '')
         return self.contentspage(
-            self.renderNesting(combos, here),
+            self.renderNesting(combos,here=here),
             singletons,
             REQUEST=REQUEST)
 
+    def referringPageId(self,REQUEST=None):
+        """
+        If the referrer was a page in this wiki, return its id, or None.
+        """
+        if not REQUEST: REQUEST = self.REQUEST
+        if not REQUEST: return None
+        referrer = REQUEST.get('HTTP_REFERER', None)
+        if not referrer: return None
+        m = re.match(r'^'+re.escape(self.wikiUrl())+r'/?([^?#/]*)', referrer)
+        if m: return m.group(1)
+        else: return None
+
+    def referringPageName(self,REQUEST=None):
+        """
+        If the referrer was a page in this wiki, return its name, or None.
+        """
+        id = self.referringPageId(REQUEST=REQUEST)
+        if id:
+            p = self.pageWithId(id)
+            if p: return p.pageName()
+        return None
+
     security.declareProtected(Permissions.View, 'context')
     def context(self, REQUEST=None, with_siblings=0, enlarge_current=0):
         """
@@ -682,7 +716,8 @@ class OutlineRendering:
             nesting, here,
             enlarge_current=enlarge_current,
             suppress_hyperlink=suppress_hyperlink)
-        # special case: if parent seems to be missing, reset XXX
+        # special case: if parent seems to be missing, reset
+        # XXX will not match ? clean up
         if hierarchy == '<ul>\n</ul>':
             self.setParents([])
             self.index_object()
@@ -810,7 +845,7 @@ class OutlineRendering:
         Format a nesting structure as HTML unordered lists of wiki links.
 
         - nesting is the nesting to be formatted
-        - here is the page name to highlight with "you are here", if any
+        - here is the name of the page to highlight with "you are here", if any
         - if enlarge_current is true, here will be enlarged instead
         - if suppress_hyperlink is true, here will not be linked
           (backwards compatibility for old editforms)
@@ -828,9 +863,9 @@ class OutlineRendering:
             # quicker than renderLinkToPage, since we know it exists:
             wikiurl = self.wiki_url()
             def quicklink(page):
+                id = self.canonicalIdFrom(page)
                 return '<a href="%s/%s" name="%s">%s</a>' \
-                       % (wikiurl,self.canonicalIdFrom(page),quote(page),
-                          self.formatWikiname(page))
+                       % (wikiurl,id,id,self.formatWikiname(page))
             if here and page == here: 
                 if enlarge_current:
                     # just assume we are in the page header, and link to
@@ -845,7 +880,7 @@ class OutlineRendering:
                              self.formatWikiname(page)))
                 else:
                     # just highlight "here"
-                    return '%s <b><-- %s.</b>' % \
+                    return '%s <span id="youarehere"><-- %s.</span>' % \
                            ((suppress_hyperlink and page) # another special case
                             or quicklink(page),
                             _("You are here"))
@@ -858,18 +893,17 @@ class OutlineRendering:
             return ''
         if did is None: did = []
         if got is None:
-            # XXX trying hard to turn off bullets in plone!
-            got = ['<ul style="list-style:none; margin-left:0;" class="aqtree3clickable">']
+            got = ['<ul class="outline expandable">']
             recursing = 0
         else:
             recursing = 1
         for n in nesting:
             if type(n) == ListType:
                 if not (n[0]==here and suppress_current): #XXX temp
-                    got.append('%s <li style="list-style:none" class="aqtree3clickable">%s' % (indent,renderContentsLink(n[0])))
+                    got.append('%s <li>%s' % (indent,renderContentsLink(n[0])))
                 if len(n) > 1:
                     if not (n[0]==here and suppress_current): #XXX temp
-                        got.append('<ul>')
+                        got.append('<ul class="outline expandable">')
                     for i in n[1:]:
                         if type(i) == ListType:
                             got = self.renderNesting(
@@ -882,7 +916,7 @@ class OutlineRendering:
                                 got=got,
                                 indent=indent+' ')
                         else:
-                            got.append('%s <li style="list-style:none" class="aqtree3clickable">%s</li>' % (indent,renderContentsLink(i)))
+                            got.append('%s <li>%s</li>' % (indent,renderContentsLink(i)))
                     if not (n[0]==here and suppress_current): #XXX temp
                         got.append("</ul>")
                 else:
@@ -890,7 +924,7 @@ class OutlineRendering:
                 if not (n[0]==here and suppress_current):
                     got.append('%s </li>' % indent)
             else:
-                got.append('%s <li style="list-style:none" class="aqtree3clickable">%s</li>' % (indent,renderContentsLink(n)))
+                got.append('%s <li>%s</li>' % (indent,renderContentsLink(n)))
 
         if recursing:
             return got
