@@ -33,6 +33,11 @@ from I18n import _
 try: from transaction import get as get_transaction
 except ImportError: get_transaction = get_transaction
 
+ZWIKI_BIRTHDATE='1999/11/05'
+
+ZWIKIDIR = os.path.dirname(__file__)
+def abszwikipath(path): return os.path.join(ZWIKIDIR,path)
+
 
 class PageUtils:
     """
@@ -124,7 +129,7 @@ class PageUtils:
         strings = filter(lambda x:type(x)==type(''), strings)
         return len(join(strings,''))
 
-    def summary(self,size=200,paragraphs=1):
+    def summary(self,size=200,paragraphs=1,charset="utf-8"):
         """
         Give a short plaintext summary of this page's content.
 
@@ -135,11 +140,13 @@ class PageUtils:
         """
         size, paragraphs = int(size), int(paragraphs)
         t = self.documentPart()
+        t = t.decode(charset)  # go to unicode.
         t = re.sub(r'<(?=\S)[^>]+>','',t).strip() # strip html tags
         if paragraphs: t = join(split(t,'\n\n')[:paragraphs],'\n\n')
         if len(t) > size:
             t = t[:size]
             t = re.sub(r'\w*$',r'',t) + '...'
+        t = t.encode(charset)  # get back from unicode.
         return html_quote(t)
 
     def renderedSummary(self,size=500,paragraphs=1):
@@ -327,15 +334,24 @@ class PageUtils:
     def contentsUrl(self, scroll=1):
         """
         Return the url of zwiki's contents method.
-        
-        By default we put the current page name as a target and scroll
-        there, note in a large public wiki that generates a lot of unique
-        urls for robots, if robots are dumb enough to crawl both
-        url#target1 and url#target2.
-        """
-        return self.defaultPageUrl() + '/contents' + \
-               (scroll and ('#%s' % quote(self.pageName())) or '')
 
+        In general, we try to keep these urls stable, so as to minimise
+        useless work done for web robots. For the contents page, this was
+        tricky because we like it to know what page we were looking at
+        (for you are here), and to scroll there. Here's what we do now:
+
+        - use front page url as our fixed base
+
+        - add #PageId so the browser will scroll; most robots ignore
+          this part, we think
+
+        - have the contents page figure out"you are here" from the http referer
+
+        """
+        url = self.defaultPageUrl() + '/contents'
+        if scroll: url += '#' + self.pageId()
+        return url
+    
     security.declareProtected(Permissions.View, 'changesUrl')
     def changesUrl(self):
         return self.urlForDtmlPageOrMethod('RecentChanges','recentchanges')
@@ -359,7 +375,7 @@ class PageUtils:
 
     security.declareProtected(Permissions.View, 'helpUrl')
     def helpUrl(self):
-        return self.urlForPageOrDefault('HelpPage','')
+        return self.urlForDtmlPageOrMethod('HelpPage','helppage')
 
     security.declareProtected(Permissions.View, 'searchUrl')
     def searchUrl(self):
@@ -372,7 +388,12 @@ class PageUtils:
         """
         try: return DateTime(self.creation_time)
         except (AttributeError,DateTimeSyntaxError):
-            return DateTime('2001/1/1')
+            # if the time is corrupt or missing somehow, don't beat
+            # around the bush; be darn sure to give it some fixed time
+            # or we'll see repeats in rss feeds & planets.
+            # folder's creation time would be good but that's not
+            # available.
+            return DateTime(ZWIKI_BIRTHDATE)
 
     security.declareProtected(Permissions.View, 'lastEditTime')
     def lastEditTime(self):
@@ -381,7 +402,8 @@ class PageUtils:
         """
         try: return DateTime(self.last_edit_time)
         except (AttributeError,DateTimeSyntaxError):
-            return DateTime('2001/1/1')
+            # similar considerations to creationTime()
+            return DateTime(ZWIKI_BIRTHDATE)
 
     security.declareProtected(Permissions.View, 'folder')
     def folder(self):
@@ -514,20 +536,12 @@ InitializeClass(PageUtils)
 
 
 # generic utilities
-def BLATHER(*args):
-    tmp = []
-    for arg in args: tmp.append(str(arg))
-    zLOG.LOG('ZWiki',zLOG.BLATHER,' '.join(tmp))
 
-def WARN(*args):
-    tmp = []
-    for arg in args: tmp.append(str(arg))
-    zLOG.LOG('ZWiki',zLOG.WARNING,' '.join(tmp))
-
-def DEBUG(*args):
-    tmp = []
-    for arg in args: tmp.append(str(arg))
-    zLOG.LOG('ZWiki',zLOG.DEBUG,' '.join(tmp))
+#logging
+def STDERR(*args):  sys.stderr.write(' '.join(map(str,args)) + '\n')
+def BLATHER(*args): zLOG.LOG('ZWiki',zLOG.BLATHER,' '.join(map(str,args)))
+def WARN(*args):    zLOG.LOG('ZWiki',zLOG.WARNING,' '.join(map(str,args)))
+def DEBUG(*args):   zLOG.LOG('ZWiki',zLOG.DEBUG,' '.join(map(str,args)))
 
 def formattedTraceback():
     type,val,tb = sys.exc_info()
