@@ -64,13 +64,14 @@ import Permissions
 from Defaults import AUTO_UPGRADE, IDS_TO_AVOID, \
      PAGE_METATYPE, LINK_TO_ALL_CATALOGED, LINK_TO_ALL_OBJECTS, \
      WIKINAME_LINKS, BRACKET_LINKS, DOUBLE_BRACKET_LINKS, \
-     DOUBLE_PARENTHESIS_LINKS
+     DOUBLE_PARENTHESIS_LINKS, ISSUE_LINKS, \
+     CONDITIONAL_HTTP_GET, CONDITIONAL_HTTP_GET_IGNORE
 from Regexps import url, bracketedexpr, singlebracketedexpr, \
      doublebracketedexpr, doubleparenthesisexpr, wikiname, wikilink, \
      interwikilink, remotewikiurl, protected_line, zwikiidcharsexpr, \
      anywikilinkexpr, markedwikilinkexpr, localwikilink, \
      spaceandlowerexpr, dtmlorsgmlexpr, wikinamewords, hashnumberexpr
-from Utils import PageUtils, BLATHER
+from Utils import PageUtils, BLATHER, DateTimeSyntaxError
 from Views import PageViews
 from OutlineSupport import PageOutlineSupport
 from Diff import PageDiffSupport
@@ -190,12 +191,11 @@ class ZWikiPage(
     def pageType(self):
         """Return this page's page type object."""
         # check for page type problems
-        # I'd rather do this in upgrade(), but need to make
-        # sure the page is always viewable
         # old-style page type string ?
         if type(self.page_type) == StringType:
-            BLATHER('upgraded page_type attribute of '+self.id())
-            self.setPageType(self.page_type)
+            new = self.newPageTypeIdFor(self.page_type)
+            BLATHER("upgrading %s's page type from %s to %s" % (self.id(),self.page_type,new))
+            self.setPageType(new)
         # page type whose plugin is no longer installed ?
         elif not hasattr(self.page_type,'render'):
             self.setPageType(DEFAULT_PAGETYPE)
@@ -204,7 +204,7 @@ class ZWikiPage(
         """Return the page type object with this id (or the default)"""
         match = filter(lambda x:x._id==id,PAGETYPES)
         return (match and match[0]) or DEFAULT_PAGETYPE
-    security.declarePublic('pageType') # useful for troubleshooting
+    security.declarePublic('pageTypeId') # useful for troubleshooting
     def pageTypeId(self):
         """Return the short id for this page's page type."""
         return self.pageType().id()
@@ -247,6 +247,8 @@ class ZWikiPage(
         """
         Render this zwiki page, also upgrading it on the fly if needed.
         """
+        if self.handle_modified_headers(REQUEST=REQUEST):
+            return '' # return a 304 response with no content
         if AUTO_UPGRADE: self.upgrade(REQUEST)
         rendered = self.render(client,REQUEST,RESPONSE,**kw)
         return rendered
@@ -294,7 +296,7 @@ class ZWikiPage(
         if hasattr(self,'_v_cooked'):
             delattr(self,'_v_cooked')
             delattr(self,'_v_blocks')
-        if REQUEST: REQUEST.RESPONSE.redirect(self.page_url())
+        if REQUEST: REQUEST.RESPONSE.redirect(self.pageUrl())
 
     def cookDtmlIfNeeded(self):
         if self.dtmlAllowed() and self.hasDynamicContent(): self.cook()
@@ -316,8 +318,13 @@ class ZWikiPage(
         # optimization: to save memory, avoid unnecessarily calling DTML
         # and generating _v_blocks data
         if not self.hasDynamicContent(): return self.preRendered()
-        return DTMLDocument.__call__(self,client,REQUEST,RESPONSE,**kw)
-
+        return DTMLDocument.__call__(
+            self.__of__(self.folder()), # ensure dtml in pages can acquire
+            client,
+            REQUEST,
+            RESPONSE,
+            **kw)
+    
     def renderMidsectionIn(self, text, **kw):
         """
         Insert some final things between the rendered document and discussion.
@@ -422,13 +429,74 @@ class ZWikiPage(
         """Is Epoz installed ?"""
         return hasattr(self,'Epoz')
 
-    ######################################################################
+    def handle_modified_headers(self, last_mod=None, REQUEST=None):
+        """
+        Check if the headers indicate we have changed content.
+
+        Return True if nothing changed, False otherwise. Set Headers as needed.
+        Methods using this should call this before returning any content,
+        then if a 304 is called for this method returns True and
+        the calling method should give no content to the browser.
+        """
+        RESPONSE=REQUEST.RESPONSE
+        # do we handle things at all?
+        if not getattr(self, 'conditional_http_get', CONDITIONAL_HTTP_GET):
+            return False
+        # admins can specify a list of property names that make us ignore
+        # "Conditional HTTP Get" processing if they are set
+        # especially useful for ignoring pages with allow_dtml
+        ignore = getattr(self, 'conditional_http_get_ignore', \
+                                        CONDITIONAL_HTTP_GET_IGNORE)
+        for ignore_property in ignore:
+            if getattr(self, ignore_property, False): return False
+        if last_mod == None:
+            try:
+                # bobobase_modification_time reflects also changes
+                # to voting, not like last_edit_time
+                last_mod = self.bobobase_modification_time()
+            except DateTimeSyntaxError:
+                # if anything goes wrong with the stored date, we just
+                # ignore all 304 handling and go on as if nothing happened
+                BLATHER("invalid bobobase_modification time in page %s" \
+                            % (self.id()))
+                return False
+        try: # we could have been fed an illegal date string
+            last_mod = long(DateTime(last_mod).timeTime())
+        except DateTimeSyntaxError:
+            BLATHER("invalid date input on page %s" % (self.id()))
+            return False
+        header=REQUEST.get_header('If-Modified-Since', None)
+        if header is not None:
+            header=header.split( ';')[0]
+            # Some proxies seem to send invalid date strings for this
+            # header. If the date string is not valid, we ignore it
+            # rather than raise an error to be generally consistent
+            # with common servers such as Apache (which can usually
+            # understand the screwy date string as a lucky side effect
+            # of the way they parse it).
+            # This happens to be what RFC2616 tells us to do in the face of an
+            # invalid date.
+            try:   mod_since=long(DateTime(header).timeTime())
+            except DateTimeSyntaxError: mod_since=None
+            if mod_since is not None:
+                if last_mod > 0 and last_mod <= mod_since:
+                    RESPONSE.setHeader('Last-Modified',
+                                       rfc1123_date(last_mod))
+                    RESPONSE.setStatus(304)
+                    return True
+        RESPONSE.setHeader('Last-Modified', rfc1123_date(last_mod))
+        return False
+
     # link rendering and handling
 
     def wikinameLinksAllowed(self):
         """Are wikinames linked in this wiki ?"""
         return getattr(self,'use_wikiname_links',WIKINAME_LINKS)
 
+    def issueLinksAllowed(self):
+        """Are issue numbers (#NNNN) linked in this wiki ?"""
+        return getattr(self,'use_issue_links',ISSUE_LINKS)
+
     def bracketLinksAllowed(self):
         """Are bracketed freeform names linked in this wiki ?"""
         return getattr(self,'use_bracket_links',BRACKET_LINKS)
@@ -450,8 +518,10 @@ class ZWikiPage(
         """Does link look a valid wiki link syntax for this wiki ?
         """
         return ((
-            self.wikinameLinksAllowed() and
-                re.match(wikiname,link)
+            (self.wikinameLinksAllowed() and
+                re.match(wikiname,link))
+            or (self.issueLinksAllowed() and
+                re.match(hashnumberexpr,link))
             or (self.bracketLinksAllowed() and
                 re.match(singlebracketedexpr,link))
             or (self.doubleBracketLinksAllowed() and
@@ -483,7 +553,6 @@ class ZWikiPage(
                 if (link[0]=='!'
                     or not (self.isValidWikiLinkSyntax(link)
                             or (urls and re.match(url,link))
-                            or re.match(hashnumberexpr,link)
                             )
                     or within_literal(linkstart,linkend-1,state,text) # XXX these
                     or withinSgmlOrDtml((linkstart,linkend),text)):   # overlap ?
@@ -566,6 +635,11 @@ class ZWikiPage(
         if link[0] == '!':
             return link[1:]
 
+        # ignore certain WikiNames from a property ('Lines' type property)
+        ignore_wikinames = getattr(self, 'ignore_wikinames', [])
+        if link in ignore_wikinames:
+            return link
+
         # is it an interwiki link ?
         if re.match(interwikilink,link):
             return self.renderInterwikiLink(link)
@@ -576,7 +650,7 @@ class ZWikiPage(
             if re.search(
                 r'(?s)<a name="ref%s"' % (re.escape(linknobrackets)),text):
                 return '<a href="%s#ref%s" title="footnote %s">[%s]</a>' % (
-                    self.page_url(),linknobrackets,
+                    self.pageUrl(),linknobrackets,
                     linknobrackets,linknobrackets)
 
         # is it a bare URL ?
@@ -590,7 +664,7 @@ class ZWikiPage(
             p = self.issuePageWithNumber(self.issueNumberFrom(link))
             if p:
                 try: link = p.getId() # XXX poor caching
-                except: link = p.id   # all-brains
+                except AttributeError: link = p.id   # all-brains
                 return self.renderLinkToPage(link,
                                              linkorig=linkorig,
                                              link_title=link_title,
@@ -612,7 +686,7 @@ class ZWikiPage(
             p = self.pageWithFuzzyName(link)
             if p:
                 try: link = p.getId() # XXX poor caching
-                except: link = p.id   # all-brains
+                except AttributeError: link = p.id   # all-brains
                 # and fall through
             
         # must be either a WikiName link, or an existing page's id from above
@@ -638,7 +712,7 @@ class ZWikiPage(
             # make sure we have the page's id
             if not self.pageWithId(page): # XXX this check helps avoid zodb loads ?
                 try: page = p.getId() # XXX poor caching
-                except: page = p.id   # all-brains
+                except AttributeError: page = p.id   # all-brains
             title     = (link_title and ' title="%s"' % link_title) or '' #' title="%s"' % self.pageWithId(page).linkTitle()
             name      = (name and ' name="%s"' % name) or ''
             accesskey = (access_key and ' accesskey="%s"' % access_key) or ''
@@ -646,7 +720,7 @@ class ZWikiPage(
             if p.isIssue():
                 try:
                     style=' style="background-color:%s;"' % p.issueColour() # poor caching
-                except:
+                except AttributeError:
                     style=' style="background-color:%s;"' % p.issueColour # all-brains
             else:
                 style = ''
@@ -691,8 +765,9 @@ class ZWikiPage(
         if not self.isValidWikiLinkSyntax(local): return link
         local = re.sub(bracketedexpr, r'\1', local) #XXX should support ((..))
         # look for a RemoteWikiURL definition
-        if hasattr(self.folder(), local): 
-            m = re.search(remotewikiurl,getattr(self.folder(),local).text())
+        localpage = self.pageWithName(local)
+        if localpage:
+            m = re.search(remotewikiurl, localpage.text())
             if m:
                 return '<a href="%s%s">%s:%s</a>' \
                        % (m.group('remoteurl'),remote,local,remote)
@@ -788,31 +863,32 @@ class ZWikiPage(
         
         with prettyprint=1, format it for use in the standard header.
         """
-        interval = self.asAgeString(last_edit_time)
+        try:
+            interval = self.asAgeString(last_edit_time)
+        except DateTimeSyntaxError:
+            # we got fed with a non-valid date
+            interval = self.asAgeString(None)
         if not prettyprint:
             s = _("last edited %(interval)s ago") % {"interval":interval}
         else:
-            try:
-                #XXX do timezone conversion ?
-                lastlog = self.lastlog()
-                if lastlog: lastlog = ' ('+lastlog+')'
-                
-                # build the link around the interval
-                linked_interval = (
-                    ' <a href="%s/diff" title="%s (%s)" >%s</a>' % (
-                    self.pageUrl(),
-                    _('show last edit'),
-                    lastlog, 
-                    interval))
-                
-                # use the link in a clear i18n way                                                                         
-                s =  _('last edited %(interval)s ago')  % {"interval": linked_interval}
+            lastlog = self.lastlog()
+            if lastlog and len(lastlog)>0:
+                lastlog = ' ('+lastlog+')'
+
+            # build the link around the interval
+            linked_interval = (
+                ' <a href="%s/diff" title="%s%s" >%s</a>' % (
+                self.pageUrl(),
+                _('show last edit'),
+                lastlog, 
+                interval))
+
+            # use the link in a clear i18n way
+            s =  _('last edited %(interval)s ago')  % {"interval": linked_interval}
 
-            except:
-                s = _('last edited %(interval)s ago') % {"interval": interval}
-                
         if (last_editor and
-            not re.match(r'^[0-9\.\s]*$',last_editor)):
+            # anonymous? try to find out by somehow matching an ip address:
+            not re.match(r'^(?:\d{1,3}\.){3}\d{1,3}$',last_editor)):
             # escape some things that might cause trouble in an attribute
             editor = re.sub(r'"',r'',last_editor)
 
@@ -843,6 +919,9 @@ class ZWikiPage(
         to handle eg pages created via the ZMI.
         """
         return self.title_or_id()
+
+    def pageId(self):
+        return self.id()
     
     security.declareProtected(Permissions.View, 'spacedPageName')
     def spacedPageName(self):
@@ -1251,7 +1330,7 @@ class ZWikiPage(
         p = self.pageWithNameOrId(page)
         if p:
             try: id = p.getId() # poor caching
-            except: id = p.id #all-brains
+            except AttributeError: id = p.id #all-brains
         else: id = page
         if self.hasCatalogIndexesMetadata(
             (['meta_type','path','canonicalLinks'], [])):
@@ -1427,35 +1506,28 @@ def stripDelimitersFrom(link):
 
 
 # ZMI page creation form
-manage_addZWikiPageForm = DTMLFile('skins/zwiki_standard/addwikipageform', globals())
+manage_addZWikiPageForm = DTMLFile('skins/zwiki/addwikipageform', globals())
 
-def manage_addZWikiPage(self, id, title='', file='', REQUEST=None,
-                        submit=None):
+def manage_addZWikiPage(self, name, REQUEST=None):
     """
-    Add a ZWiki Page object with the contents of file.
-
-    Usually zwiki pages are created by clicking on ? (via create); this
-    allows them to be added in the standard ZMI way. These should give
-    mostly similar results; refactor the two methods together if possible.
+    Add a new ZWiki Page to the current folder, in the ZMI.
     """
-    # create page and initialize in proper order, as in create.
-    p = ZWikiPage(source_string='', __name__=id)
+    # create page
+    # this should be done in the proper order. Based on create()
+    # XXX extract common method ?
+    id = ZWikiPage().canonicalIdFrom(name)
+    p = ZWikiPage(__name__=id)
+    p.title=name
     newid = self._setObject(id,p)
     p = getattr(self,newid)
-    p.title=title
     p.setCreator(REQUEST)
     p.setLastEditor(REQUEST)
     p._setOwnership(REQUEST)
-    p.setPageType(getattr(self,'allowed_page_types',[None])[0])
-    text = file
-    if type(text) is not StringType: text=text.read()
-    p.setText(text or '',REQUEST)
-    p.wikiOutline().add(p.pageName()) # update the wiki outline
-    p.index_object()
-    if REQUEST is not None:
-        try: u=self.DestinationURL()
-        except: u=REQUEST['URL1']
-        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
-        REQUEST.RESPONSE.redirect(u+'/manage_main')
-    return ''
-
+    p.setPageType(p.defaultPageType())
+    p.setText('',REQUEST) # no harm
+    #p.index_object() # should be no need
+    p.wikiOutline().add(p.pageName())
+    if REQUEST:
+        if REQUEST.get('redirect','')=='page': u = p.pageUrl()
+        else: u = REQUEST['URL1']+'/manage_main'
+        REQUEST.RESPONSE.redirect(u)
