@@ -26,7 +26,7 @@ from Products.ZWiki.pagetypes import registerPageType
 
 class PageTypeMoin(PageTypeBase):
     _id = 'moin'
-    _name = 'MoinMoin markup'
+    _name = 'MoinMoin'
     supportsMoin = yes
     supportsWikiLinks = yes
 
@@ -1493,7 +1493,7 @@ class MoinFormatter(MoinFormatterBase):
     def endContent(self):
         try:
             cid = self._content_ids.pop()
-        except:
+        except IndexError:
             cid = 'content'
         if cid!='content':
             aid = 'bottom_%s' % (cid,)
