@@ -13,16 +13,17 @@ from Products.ZWiki.plugins import registerPlugin
 from Products.ZWiki.Defaults import registerPageMetaData
 from Products.ZWiki import Permissions
 from Products.ZWiki.Utils import BLATHER, formattedTraceback, addHook
-from Products.ZWiki.Views import loadDtmlMethod, loadPageTemplate, \
-     STANDARD_TEMPLATES
+from Products.ZWiki.Views import loadDtmlMethod, loadPageTemplate, TEMPLATES
+     
 from Products.ZWiki.I18n import _
 
-STANDARD_TEMPLATES.update({
+TEMPLATES.update({
     'issuepropertiesform': loadDtmlMethod('issuepropertiesform','plugins/tracker'),
+    # page template wrappers
     'issuetracker'       : loadPageTemplate('issuetracker','plugins/tracker'),
     'issuebrowser'       : loadPageTemplate('issuebrowser','plugins/tracker'),
     'filterissues'       : loadPageTemplate('filterissues','plugins/tracker'),
-    # dtml versions which the above will usually use
+    # the DTML implementations
     'IssueTracker'       : loadDtmlMethod('IssueTracker','plugins/tracker'),
     'IssueBrowser'       : loadDtmlMethod('IssueBrowser','plugins/tracker'),
     'FilterIssues'       : loadDtmlMethod('FilterIssues','plugins/tracker'),
@@ -222,47 +223,62 @@ class PluginTracker:
                     category='', severity='', status='', REQUEST=None,
                     sendmail=1):
         """
-        Convenience method for creating an issue page.
+        Create a page representing an issue with the specified details.
 
-        Security notes: create will check for page creation permission.
-        Sets title/category/severity/status properties without requiring
-        Manage properties permission.
+        Security notes: sets title/category/severity/status properties
+        without requiring Manage properties permission.
+        create() will also check the edits_need_username property and
+        redirect us to /denied if appropriate.
 
-        XXX only old issue tracker dtml pages call this, AFAIK.  
-        Clean up args. pageid is really pagename.  title is not used.
-
-        We will try to place the issue under a suitable parent - the
+        We try to place the issue under a suitable parent - the
         IssueTracker page if it exists, or at the top level to avoid
         having issues scattered everywhere. Better ideas ?
+
+        Returns the new page name or None.
+
+        XXX clean up old args kept for backwards compatibility. Only
+        really old issue tracker dtml pages call this directly.
+        pageid is really pagename.  title is not used.
         """
         # XXX hardcoded.. cf trackerUrl
         if self.pageWithName('IssueTracker'): parents = ['IssueTracker']
         else: parents = []
-        self.create(pageid,text=text,REQUEST=REQUEST,parents=parents,
-                    sendmail=sendmail)
-        issue = self.pageWithName(pageid)
-        issue.manage_addProperty('category','issue_categories','selection')
-        issue.manage_addProperty('severity','issue_severities','selection')
-        issue.manage_addProperty('status','issue_statuses','selection')
-        issue.manage_changeProperties(title=pageid,
-                                      category=category,
-                                      severity=severity,
-                                      status=status
-                                      )
-        self.index_object()
+        # might fail due to edits_need_username
+        name = self.create(pageid,text=text,REQUEST=REQUEST,parents=parents,
+                           sendmail=sendmail)
+        if name:
+            issue = self.pageWithName(pageid)
+            issue.manage_addProperty('category','issue_categories','selection')
+            issue.manage_addProperty('severity','issue_severities','selection')
+            issue.manage_addProperty('status','issue_statuses','selection')
+            issue.manage_changeProperties(title=pageid,
+                                          category=category,
+                                          severity=severity,
+                                          status=status
+                                          )
+            #self.index_object() # XXX why ??
+            return name
+        else:
+            return None
 
     security.declareProtected(Permissions.Add, 'createNextIssue')
     def createNextIssue(self,name='',text='',category='',severity='',status='',
                         REQUEST=None,sendmail=1):
         """
-        Create a new issue page, using the next available issue number.
+        Add a new issue page, using the next available number, and redirect.
 
-        Returns the page name that was used.
+        createIssue() (really create()) will also check the
+        edits_need_username property and may redirect to an error.
+        Otherwise this will redirect to the tracker url (when called
+        via the web). Returns the new page's name or None.
         """
         newnumber = self.nextIssueNumber(REQUEST=REQUEST)
         pagename=self.pageNameFromIssueNumberAndName(newnumber,name)
-        self.createIssue(pagename, text, None, 
-                         category, severity, status, REQUEST,sendmail)
+        pagename=self.createIssue(pagename,text,None, 
+                                  category,severity,status,REQUEST,sendmail)
+        if pagename:
+            if REQUEST:
+                REQUEST.RESPONSE.redirect(self.trackerUrl())
         return pagename
 
     security.declareProtected(Permissions.View, 'nextIssueNumber')
@@ -316,6 +332,7 @@ class PluginTracker:
                     raise 'Unauthorized', (_('You are not authorized to rename this ZWiki Page.'))
                 self.rename(newpagename, updatebacklinks=1, sendmail=0,
                             REQUEST=REQUEST)
+ 
         if category:
             old = getattr(self,'category','')
             if category != old:
@@ -356,7 +373,7 @@ class PluginTracker:
             #self.setLastEditor(REQUEST)
             #self.reindex_object()
         if REQUEST:
-            REQUEST.RESPONSE.redirect(self.page_url())
+            REQUEST.RESPONSE.redirect(self.pageUrl())
 
     def category_index(self):
         """helper method to facilitate sorting catalog results"""
@@ -466,7 +483,6 @@ class PluginTracker:
         """
         Render the issue properties form as a (customizable) HTML fragment.
         """
-        #return onlyBodyFrom(
         return self.getSkinTemplate('issuepropertiesform')(self,REQUEST)
 
     security.declareProtected(Permissions.View, 'trackerUrl')
@@ -508,7 +524,7 @@ class PluginTracker:
             'severity_index',
             'status',
             'status_index',
-            ]
+            'issueName']
         KeywordIndexes = [
             ]
         DateIndexes = [
@@ -545,21 +561,20 @@ class PluginTracker:
                 if not self.pageWithName(page):
                     self.create(page,
                                 text=open(os.path.join(dir,page+'.dtml'),'r').read(),
-                                sendmail=0)
+                                sendmail=0,
+                                type='stx')
+
             # also, disable subtopics display under IssueTracker
             self.IssueTracker.setSubtopicsPropertyStatus(0)
+
         # index each page, to make all indexes and metadata current
         # may duplicate some work in setupCatalog
         n = 0
         cid = self.catalog().getId()
         for p in self.pageObjects():
             n = n + 1
-            try:
-                BLATHER('indexing page #%d %s in %s'%(n,p.id(),cid))
-                p.index_object(log=0)
-            except:
-                BLATHER('failed to index page #%d %s: %s' \
-                        % (n,p.id(),formattedTraceback()))
+            BLATHER('indexing page #%d %s in %s'%(n,p.id(),cid))
+            p.index_object(log=0)
         BLATHER('indexing complete, %d pages processed' % n)
         # and a dummy issue to enable site navigation links
         if not self.hasIssues():
@@ -611,6 +626,7 @@ class PluginTracker:
                     self.manage_addProperty(prop,values,'selection')
                     if default: setattr(self,prop,default)
                     changed = 1
+
         return changed
 
 InitializeClass(PluginTracker)
